from wedeliver_core_plus.helpers.kafka_producer import Producer
from wedeliver_core_plus.helpers.topics import Topics

def send_mail(emails, message, title):
    response = dict(
        body=message,
        title=title,
        emails=emails
    )

    Producer().send_topic(Topics().SEND_MAIL, response)


import traceback
from functools import wraps
from sqlalchemy.exc import SQLAlchemyError

from wedeliver_core_plus import WedeliverCorePlus
from wedeliver_core_plus.helpers.auth import Auth
from wedeliver_core_plus.helpers.format_exception import format_exception
from wedeliver_core_plus.helpers.kafka_producers.notification_center import send_critical_error

from wedeliver_core_plus.helpers.validate_parameters import (
    validate_parameters,
)


def handle_exceptions(func):
    app = WedeliverCorePlus.get_app()

    @wraps(func)
    def inner_function(*args, **kws):
        try:
            result = func(*args, **kws)
            return result

        except Exception as e:
            # If the error related to the Database, then close the database session if it is open
            if isinstance(e, SQLAlchemyError):
                try:
                    db = app.extensions['sqlalchemy'].db
                    db.session.close()
                except Exception:
                    pass

            notification_channel = None
            if func is not None:
                if not validate_parameters(function=func):
                    notification_channel = "empty-parameters"

            use_default_response_message_key = True
            if hasattr(e, "custom_exception"):
                public_message = e.args[0] if e.args else e.message if hasattr(e, 'message') else 'Unknown'
                status_code = e.code
                use_default_response_message_key = (
                    e.use_default_response_message_key
                    if hasattr(e, 'use_default_response_message_key') else True
                )
                send_notification = hasattr(e, "silent") and not e.silent
                notification_channel = "soft-errors"
            else:
                send_notification = True
                public_message = "Unhandled Exception"
                status_code = 500

            message = format_exception(
                exception=traceback.format_exc(),
                user=Auth.get_user(),
                status_code=status_code
            )
            app.logger.error(message)
            if send_notification:
                send_critical_error(message=message, channel=notification_channel)

            return public_message, status_code, use_default_response_message_key

    return inner_function



from functools import wraps

from flask import request

from wedeliver_core_plus.helpers.auth import verify_user_token_v2, Auth
from wedeliver_core_plus.helpers.exceptions import (
    AppValidationError,
    AppMissingAuthError, AppForbiddenError,
)
from flask_babel import  _


def handle_auth(require_auth, append_auth_args=None, allowed_roles=None, pre_login=False, allowed_permissions=None,guards=[]):
    def factory(func):
        @wraps(func)
        def inner_function(*args, **kws):
            # user_language = Auth.get_user_language()
            # with force_locale(user_language):
            if not require_auth:
                return func(*args, **kws)

            if "Authorization" not in request.headers:
                raise AppMissingAuthError(_("Missing authentication"))

            token = request.headers["Authorization"]
            if "country_code" not in request.headers and request.endpoint != "health_check":
                raise AppValidationError(_("Country Code is Required (c)"))

            user = verify_user_token_v2(token=token)

            # with force_locale(user.get("language")):
            if not pre_login:
                if not user.get("is_logged"):
                    raise AppValidationError(_("Not Logged Token, please complete login process"))

            if allowed_roles:
                if user.get("role") not in allowed_roles:
                    raise AppValidationError(_("Not Allowed Role"))

            if guards:
                for guard in guards:
                    if not guard():
                        raise AppForbiddenError("Not Allowed Feature")


            if append_auth_args and isinstance(append_auth_args, list):
                for arg in append_auth_args:
                    if not kws.get('appended_kws'):
                        kws['appended_kws'] = dict()
                    if '.' in arg:

                        if 'as' in arg:
                            arg, as_arg = arg.split(' as ')
                        else:
                            as_arg = arg.replace('.', '_')

                        obj, key = arg.split('.')
                        value = user.get(obj, {}).get(key)

                        kws['appended_kws'][as_arg] = value
                    else:
                        value = user.get(arg)
                        kws['appended_kws'][arg] = value

            return func(*args, **kws)

        return inner_function

    return factory



from wedeliver_core_plus import WedeliverCorePlus
from wedeliver_core_plus.helpers.amazon.get_s3_client import get_s3_client


def upload_file(file_binary, file_name, content_type, s3_bucket=None, s3_client=None):
    """Upload a file to an S3 bucket"""
    app = WedeliverCorePlus.get_app()
    # Upload the file
    if not s3_client:
        s3_client = get_s3_client()

    result = s3_client.upload_fileobj(
        file_binary,
        s3_bucket if s3_bucket else app.config.get("S3_BUCKET"),
        file_name,
        ExtraArgs={"ContentType": content_type},
    )
    return result



# -*- coding: utf-8 -*-

import sys
import logging
from dotenv import load_dotenv

load_dotenv()


class AppLogger:
    stream = None
    level = None

    def __init__(self, level, stream=None):
        self.stream = stream or sys.stdout
        self.level = level

    def get_logger(self):
        handler = logging.StreamHandler(self.stream)
        handler.setLevel(self.level)
        handler.addFilter(LevelFilter(self.level))
        formatter = logging.Formatter(
            "(%(levelname)s) %(name)s in %(module)s@%(lineno)s: %(message)s"
        )

        handler.setFormatter(formatter)

        return handler


class LevelFilter(logging.Filter):
    def __init__(self, level):
        super().__init__()
        self.level = level

    def filter(self, record):
        return record.levelno == self.level


handler_list = [
    AppLogger(level=logging.CRITICAL, stream=sys.stderr).get_logger(),
    AppLogger(level=logging.ERROR, stream=sys.stderr).get_logger(),
    AppLogger(level=logging.WARNING).get_logger(),
    AppLogger(level=logging.INFO).get_logger(),
]


def init_logger(app):
    if app.config.get("DEBUG"):
        handler_list.append(AppLogger(level=logging.DEBUG).get_logger())

    packages_loggers = [
        dict(app=logging.getLogger(), start_level=logging.DEBUG),  # root
        dict(app=app.logger, start_level=logging.DEBUG),  # flask
        dict(
            app=logging.getLogger("werkzeug"),  # webserver
            start_level=logging.DEBUG
            if "production" not in app.config.get("FLASK_ENV")
            else logging.WARNING,
        ),
    ]
    for package_logger in packages_loggers:
        logger = package_logger.get("app")
        start_level = package_logger.get("start_level")
        logger.setLevel(start_level)  # start listen form debug level
        logger.propagate = False
        for handler in logger.handlers:
            logger.removeHandler(handler)

        for handler in handler_list:
            logger.addHandler(handler)



import jwt
import hashlib
from flask import request
from datetime import datetime
from app import app
from wedeliver_core_plus.helpers.exceptions import AppValidationError


def authenticate_token(token, force_expiration_datetime):
    """
    This function is used to authenticate the user token
    """

    try:
        verify_token = app.config.get("FLASK_ENV") == "production"
        token_result = jwt.decode(
            token,
            key=app.config.get("SECRET_KEY"),
            algorithms=["HS256"],
            do_verify=verify_token,
        )
        # user_role = token_result.get("role")
        # Test

        if request.headers.get("Accept-Language") in ("ar", "en"):
            token_result["language"] = request.headers.get("Accept-Language")
        else:
            token_result["language"] = token_result.get("language", "ar")

        user_app = token_result.get("app")

        if not token_result.get("created_at"):
            raise jwt.ExpiredSignatureError()

        token_created_at = datetime.strptime(
            token_result.get("created_at"), "%Y-%m-%d %H:%M:%S"
        )
        for rule_app, expire_datetime, message in force_expiration_datetime:
            if rule_app == user_app and token_created_at < expire_datetime:
                raise jwt.ExpiredSignatureError(message)

        return dict(message="Valid User", data=token_result)

    except jwt.DecodeError:
        raise AppValidationError("Token is not valid")
    except jwt.ExpiredSignatureError:
        raise AppValidationError("Token is expired")


def hash_password(password):
    # """Hash a password for storing."""
    hashed_password = hashlib.sha256(password.encode("utf-8")).hexdigest()
    return hashed_password


def verify_password(stored_password, provided_password):
    hashed_plaintext_password = hashlib.sha256(
        provided_password.encode("utf-8")
    ).hexdigest()
    return hashed_plaintext_password == stored_password



import inspect
from datetime import datetime
from sys import platform
from wedeliver_core_plus import WedeliverCorePlus
from flask import request


def format_exception(exception, user=None, status_code=None, soft_message=None):
    app = WedeliverCorePlus.get_app()
    user = user or dict()
    caller_frame = None
    user_id = None
    token = None
    user_agent = None
    role = None
    node = None
    args = None
    ip = None
    data = None

    try:
        caller_frame = request.path
    except Exception:
        try:
            caller_frame = inspect.currentframe()
            caller_frame = inspect.getouterframes(caller_frame, 2)[1][3]
        except Exception:
            pass

    try:
        ip = request.headers.get('X-Real-IP') or request.headers.get('X-Forwarded-For') or request.remote_addr
    except Exception:
        pass

    try:
        user_agent = request.headers.get('User-Agent')
    except Exception:
        pass

    try:
        args = str(request.args.to_dict())[:1000]
    except Exception:
        pass

    content_type = request.headers.get('Content-Type')
    if content_type and 'application/json' in content_type:
        # if the request have json payload, the user need to send the Content-Type as application/json
        try:
            data = str(request.json)[:1000]
        except Exception:
            pass
    else:
        try:
            data = str(request.form.to_dict())[:1000]
        except Exception:
            pass

    try:
        if user:
            user_id = "{0} - {1} - {2}".format(
                user.get("email"),
                user.get("mobile"),
                user.get("user_id"),
            )
    except Exception:
        pass

    try:
        role = user.get("role")
    except Exception:
        pass

    try:
        node = platform if type(platform) is str else str(platform.node())
    except Exception:
        pass

    if "Authorization" in request.headers:
        token = request.headers["Authorization"]
    else:
        pass

    return """Node: {node}
Time: {_time}
Env: {env}
IP: {ip}
API: {call}
User: {user}
Token: {token}
User Agent: {user_agent}
Role: {role}
Args: {args}
Data: {data}
Status Code: {status_code}
Soft Message: {soft_message}
Error: {error}""".format(
        env=app.config.get("FLASK_ENV") or "Unknown",
        call=caller_frame or "Unknown",
        user=user_id or "Unknown",
        token=token or "Unknown",
        user_agent=str(user_agent) or "Unknown",
        role=role or "Unknown",
        node=node or "Unknown",
        _time=str(datetime.now()),
        ip=ip or "Unknown",
        args=args or "Unknown",
        data=data or "Unknown",
        error=exception or "Unknown",
        status_code=status_code or "Unknown",
        soft_message=soft_message or "Unknown",
    )





class Permission():
    # the below should be unique for each permission
    admin_permission = "admin_permission" # TODO this is will be removed later, we added to test the admin role
    # invoice
    invoice_create = "invoice_create"
    invoice_reviewer = "invoice_reviewer"

    #report
    report_create = "report_create"
    report_reviewer = "report_reviewer"



class OpsRole():
    permission = [Permission.invoice_create]

class FinanceRole():
    permission = [Permission.invoice_reviewer]


class adminRole():
    #this to add all the above permissions into the adminRole as a list of permissions
    def props(cls):
        return [i for i in cls.__dict__.keys() if i[:1] != '_']

    permission = props(Permission)




print(adminRole.permission)
#TODO
# 1
#  need to add the roles to the database later


from flask import request, g

from flask import session

from .micro_fetcher import MicroFetcher


class Auth:
    def __init__(self):
        pass

    @staticmethod
    def set_user(user):
        """
        Set user in flask global object and session
        """
        g.user=user # store user in flask global object
        session["user"] = user

    @staticmethod
    def get_user():
        default_user_str = 'administrator@wedeliverapp.com'
        try:
            user = session.get("user", dict())
        except Exception:
            user = dict(user_id=default_user_str, email=default_user_str)

        return user

    @staticmethod
    def get_user_language():
        user = Auth.get_user()

        return user.get('language', 'en')

    @staticmethod
    def get_user_str():
        # app = WedeliverCorePlus.get_app()
        # with app.test_request_context():
        user = Auth.get_user()

        if user.get('email'):
            return user.get('email')
        else:
            return "Account-{}".format(
                user.get("account_id")
            )


def verify_user_token_v2(token):
    results = MicroFetcher(
        "AUTH_SERVICE"
    ).from_function(
        "app.business_logic.auth.authenticate.authenticate"
    ).with_params(
        token=token
    ).fetch()

    results["data"].update(token=token)
    user = results["data"]

    # get language form accept language in header of request
    try:
        language = (
            request.headers["Accept-Language"].lower()
            if (
                    "Accept-Language" in request.headers
                    and request.headers["Accept-Language"] in ["en", "ar"]
            )
            else (user.get("language") or 'ar')
        )
    except Exception:
        language = 'ar'

    user["language"] = language

    Auth.set_user(user)

    return user



from wedeliver_core_plus.helpers.kafka_producer import Producer


def send_topic(topic, datajson, kafka_configs=None):
    if kafka_configs is None:
        kafka_configs = {}

    kafka_configs.update(
        producer_version=2
    )
    Producer().send_topic(topic=topic, datajson=datajson, **kafka_configs)



from setuptools import setup, find_namespace_packages

# reading long description from file
# with open('DESCRIPTION.txt') as file:
#     long_description = file.read()


# specify requirements of your package here
REQUIREMENTS = [
    "SQLAlchemy",
    "Flask-SQLAlchemy",
    "flask-marshmallow",
    "confluent-kafka",
    "requests",
    "python-dotenv",
]

# Some more details
CLASSIFIERS = [
    "Development Status :: 4 - Beta",
    "Intended Audience :: Developers",
    "Topic :: Internet",
    "License :: OSI Approved :: MIT License",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.3",
    "Programming Language :: Python :: 3.4",
    "Programming Language :: Python :: 3.5",
    "Programming Language :: Python :: 3.6",
]

setup(
    name="wedeliver_core_plus",
    version="0.3.3",
    description="wedeliver_core_plus package",
    long_description="""# Markdown supported!\n\n* wedeliver core plus\n* List of features\n""",
    long_description_content_type="text/markdown",
    url="https://www.wedeliverapp.com/",
    author="Eyad Farra",
    author_email="info@wedeliverapp.com",
    license="MIT",
    packages=find_namespace_packages(
        include=["wedeliver_core_plus", "wedeliver_core_plus.*"]
    ),
    classifiers=CLASSIFIERS,
    install_requires=REQUIREMENTS,
)
